﻿
using System.Net.Sockets;
using System.Text;
using GatewaySCO;
using Serilog;

public class CHECPrueba
{
    private const string GatewayServer = "127.0.0.1";
    private const int GatewayPort = 6690;

    public static async Task Main(string[] args)
    {
        Log.Logger = new LoggerConfiguration()
            .WriteTo.Console()
            .CreateLogger();

        try
        {
            using TcpClient client = new TcpClient();
            await client.ConnectAsync(GatewayServer, GatewayPort);
            Log.Information("Conectado al Gateway.");

            using NetworkStream stream = client.GetStream();
            
            int messageNumber = 1;
            while (true)
            {
                // Remitir un mensaje numerado al Gateway
                string message = $"Mensaje {messageNumber} desde CHEC";
                byte[] messageBytes = Encoding.UTF8.GetBytes(message);
                byte[] lengthBytes = BitConverter.GetBytes(messageBytes.Length);

                if (BitConverter.IsLittleEndian)
                {
                    Array.Reverse(lengthBytes);
                }

                byte[] bytesMsj = Util.ConcatenaArreglosBytes(lengthBytes, messageBytes);

                // await stream.WriteAsync(lengthBytes, 0, 4);
                // await stream.WriteAsync(messageBytes, 0, messageBytes.Length);
                await stream.WriteAsync(bytesMsj, 0, bytesMsj.Length);
                Log.Information($"Mensaje {messageNumber} enviado al Gateway: {message}");

                // Esperar por una o más respuestas
                await ReceiveResponsesAsync(stream);

                messageNumber++;
                await Task.Delay(5000); // Esperar 5 segundos antes de enviar el siguiente mensaje
            }
        }
        catch (Exception ex)
        {
            Log.Error(ex, "Error en CHECPrueba");
        }
        finally
        {
            Log.CloseAndFlush();
        }
    }

    private static async Task ReceiveResponsesAsync(NetworkStream stream)
    {
        byte[] lengthBuffer = new byte[4];

        while (stream.DataAvailable)
        {
            // Leer longitud del mensaje de respuesta
            int bytesRead = await stream.ReadAsync(lengthBuffer, 0, 4);
            if (bytesRead == 0) break; // La conexión se ha cerrado


            int messageLength = Util.LongitudCodificada(lengthBuffer);
            Log.Information("Mensajde Gateway longitud {longitud}", messageLength);
            byte[] messageBuffer = new byte[messageLength];
            bytesRead = await stream.ReadAsync(messageBuffer, 0, messageLength);
            if (bytesRead == 0) continue; // La conexión se ha cerrado

            string response = Encoding.UTF8.GetString(messageBuffer, 0, bytesRead);
            Log.Information($"Respuesta recibida: {response}");
        }
    }
}

