﻿using Serilog;
using Serilog.Core;
using Serilog.Events;
using Microsoft.Extensions.Configuration;
using EvaPosSCOSrv;
using EvaPosSrvAplicacionImp;
using EvaPosSrvRespImp;
using gatewaySCO.POSBC;
using GatewayCHEC.Servidor;
using Servidor.TcpGateway;

namespace GatewaySCO
{

    /// <summary>
    /// EvaPos-API : servidor api, sockets y rest.
    /// Usa Serilog para bitácora.
    /// </summary> 
    public class ProgramGatewaySCO
    {
        public static async Task Main(string[] args)
        {
            Console.WriteLine("*** Gateway SCO - Servidor: procesa peticiones de Toshiba CHEC ***");
            ProgramGatewaySCO program = new();
            program.LeeActivaConfiguracion(args);

            TcpGateway gateway = new TcpGateway();
            await gateway.Start();
        }


        public void LeeActivaConfiguracion(string[] args)
        {
            // TODO - opción de incluir la activación en la cadena de configuración.

            // Instancia objeto de configuración, usa proveedor json y variables ambiente.
            IConfigurationRoot configBuilder = new ConfigurationBuilder()
                .AddJsonFile("appsettings.json")
                .AddEnvironmentVariables()
                .Build();

            // TODO Sacar la ruta de logs del objeto de configuración adecuadamente.
            string? rutaLogs = configBuilder["Serilog:WriteTo:1:Args:path"];
            Console.WriteLine($"Archivo de logs en {rutaLogs}");

            // Instancia Serilog según parámetros en archivo de configuración.
            Log.Logger = new LoggerConfiguration()
                .ReadFrom.Configuration(configBuilder)
                .CreateLogger();

            // Lee del archivo de configuración la configuración general.
            Config config = configBuilder.GetRequiredSection("GatewayConfig").Get<Config>()
                ?? throw new ApplicationException("Archivo de configuración sin sección 'GatewayConfig'.");
            Log.Information(config.ToString());
        }

    }
}
